@@ -561,7 +561,7 @@ class NavG:
                     # Continue in the header.
                     state = STATE_HEADER
             elif state == STATE_EPOCH:
-                # Create new navigation object.
+                # Create new ephemeris object.
                 eph = EphG()
 
                 # pseudorandom noise value and timestamp of epoch (clock)
@@ -646,7 +646,7 @@ class NavG:
         ephs_mat = np.empty((K, J), dtype=EphG)
         ephs_mat[kk, jj] = ephs
 
-        # Save metadata objects.
+        # Save metadata attributes.
         self.program = program
         self.author = author
         self.created = created
@@ -660,7 +660,7 @@ class NavG:
         self.ts_bow = ts_bow
         self.len = len(t_oc)
 
-        # Save array objects.
+        # Save array attributes.
         self.t_oc = t_oc.copy()
         self.prns = prns.copy()
         self.ephs = ephs_mat.copy()
@@ -768,27 +768,25 @@ class Obs:
         Waveform factor for L2 carrier-phase measurements with a row for each of
         the `K` moments in time and a column for each of the `J` space vehicles
         found in the file.
-    Cx : (K, J) float np.ndarray
-        Possible matrix of C/A pseudoranges in meters from frequency band `x`
-        with a row for each of the `K` moments in time and a column for each of
-        the `J` space vehicles found in the file.
-    Px : (K, J) float np.ndarray
-        Possible matrix of P(Y) pseudoranges in meters from frequency band `x`
-        with a row for each of the `K` moments in time and a column for each of
-        the `J` space vehicles found in the file.
-    Lx : (K, J) float np.ndarray
-        Possible matrix of carrier-phase values in cycles from frequency band
-        `x` with a row for each of the `K` moments in time and a column for each
-        of the `J` space vehicles found in the file.
-    Dx : (K, J) float np.ndarray
-        Possible matrix of Doppler frequencies in Hz from frequency band `x`
-        with a row for each of the `K` moments in time and a column for each of
-        the `J` space vehicles found in the file.
-    Sx : (K, J) float np.ndarray
-        Possible matrix of signal strengths from frequency band `x` with a row
-        for each of the `K` moments in time and a column for each of the `J`
-        space vehicles found in the file.  The units are most likely in dB-Hz,
-        but this was not standardized and is, in fact, receiver-dependent.
+    Ax : (K, J) float np.ndarray
+        Possible matrix of obervation values, with a row for each of the `K`
+        moments in time and a column for each of the `J` space vehicles found in
+        the file.  The `A` is the observation type and can be 'C' for C/A
+        pseudoranges in meters, 'P' for P(Y) pseudoranges in meters, 'L' for
+        carrier phase in cycles, 'D' for Doppler frequency in hertz, or 'S' for
+        signal strength (likely in dB-Hz, but ultimately receiver-dependent).
+        The `x` is the frequency band (one of 1, 2, 5, 6, 7, or 8).  So,
+        altogether, `Ax` could be something like 'C1', 'L1', or 'P2'.
+    Ax_LLI : (K, J) int np.ndarray
+        Possible matrix of loss of lock indicators (0 to 7), with a row for each
+        of the `K` moments in time and a column for each of the `J` space
+        vehicles found in the file.  The `A` and `x` have the same meaning as in
+        the `Ax` attributes.
+    Ax_SSI : (K, J) int np.ndarray
+        Possible matrix of signal strength integers (0 to 9), with a row for
+        each of the `K` moments in time and a column for each of the `J` space
+        vehicles found in the file.  The `A` and `x` have the same meaning as in
+        the `Ax` attributes.
 
     Attributes, event flags
     -----------------------
@@ -870,8 +868,8 @@ class Obs:
         ts_bow = None               # timestamp of beginning of week
 
         # file-level variables
-        types_cnt = 0               # number of observation types in file
         types_list = None           # array of observation types: "L1", etc.
+        types_cnt = 0               # number of observation types in file
         types_left = 0              # observation types left for an epoch
         t_rx = None                 # receiver times of week [s] for each epoch
         T_os = None                 # receiver clock offsets [s] for each epoch
@@ -881,6 +879,8 @@ class Obs:
         wf1 = None                  # wavelength factors for any L1 signal
         wf2 = None                  # wavelength factors for any L2 signal
         obs = None                  # matrix of observations
+        LLI = None                  # matrix of loss of lock indicators
+        SSI = None                  # matrix of signal strength integers
         obs_space = 0               # rows allocated to `obs` matrix
         obs_rows = 0                # rows used in `obs` matrix
         records_left = 0            # special records left in epoch
@@ -890,11 +890,13 @@ class Obs:
         t_rx_epoch = 0.0            # receiver time of week [s]
         T_os_ep = 0.0               # receiver clock offset [s]
         event_ep = 0                # the epoch flag (integer from 0 to 6)
-        svs_ep_cnt = 0              # number of space vehicles in epoch
         svs_ep_list = [0]*99        # list of space vehicles in epoch
+        svs_ep_cnt = 0              # number of space vehicles in epoch
         svs_ep_left = 0             # space vehicles left to read in epoch
-        obs_ep_cnt = 0              # number of observations in epoch
         obs_ep_list = [0]*26*98     # list of observations in epoch
+        LLI_ep_list = [0]*26*98     # list of loss of lock indicators
+        SSI_ep_list = [0]*26*98     # list of signal strength integers
+        obs_ep_cnt = 0              # number of observations in epoch
         obs_ep_left = 0             # observations left in epoch
         wf1_ep = np.ones(98)        # template wavelength factors for GPS L1
         wf2_ep = np.ones(98)        # template wavelength factors for GPS L2
@@ -1052,6 +1054,8 @@ class Obs:
                     wf1 = np.ones(SPACE_SIZE, dtype=int)
                     wf2 = np.ones(SPACE_SIZE, dtype=int)
                     obs = np.full((SPACE_SIZE, types_cnt), FILLER)
+                    LLI = np.zeros((SPACE_SIZE, types_cnt), dtype=int)
+                    SSI = np.zeros((SPACE_SIZE, types_cnt), dtype=int)
                     obs_space = SPACE_SIZE
 
                     # Decide if more observation type labels need to be read.
@@ -1157,10 +1161,12 @@ class Obs:
                     m = 16*j
                     n_obs = obs_ep_cnt - obs_ep_left
                     obs_str = line[m:(m + 14)]
-                    if obs_str.isspace() or float(obs_str) == 0.0:
-                        obs_ep_list[n_obs] = FILLER
-                    else:
-                        obs_ep_list[n_obs] = float(obs_str)
+                    obs_ep_list[n_obs] = FILLER if obs_str.isspace() or \
+                        float(obs_str) == 0.0 else float(obs_str)
+                    LLI_ep_list[n_obs] = 0 if line[m + 14] == ' ' \
+                        else int(line[m + 14])
+                    SSI_ep_list[n_obs] = 0 if line[m + 15] == ' ' \
+                        else int(line[m + 15])
                     obs_ep_left -= 1
 
                     # Break early.  Observations for a space vehicle start on a
@@ -1187,6 +1193,10 @@ class Obs:
                             np.ones(SPACE_SIZE, dtype=int)))
                     obs = np.vstack((obs,
                             np.full((SPACE_SIZE, types_cnt), FILLER)))
+                    LLI = np.vstack((LLI,
+                            np.zeros((SPACE_SIZE, types_cnt), dtype=int)))
+                    SSI = np.vstack((SSI,
+                            np.zeros((SPACE_SIZE, types_cnt), dtype=int)))
                     obs_space += SPACE_SIZE
 
                 # Store the time, space vehicle label, and observations.
@@ -1201,6 +1211,8 @@ class Obs:
                         wf1[obs_rows] = wf1_ep[prn - 1]
                         wf2[obs_rows] = wf2_ep[prn - 1]
                     obs[obs_rows, :] = obs_ep_list[ma:mb]
+                    LLI[obs_rows, :] = LLI_ep_list[ma:mb]
+                    SSI[obs_rows, :] = SSI_ep_list[ma:mb]
                     obs_rows += 1
                     ma = mb
                     mb += types_cnt
@@ -1255,10 +1267,14 @@ class Obs:
         wf1 = wf1[:obs_rows]
         wf2 = wf2[:obs_rows]
         obs = obs[:obs_rows, :]
+        LLI = LLI[:obs_rows, :]
+        SSI = SSI[:obs_rows, :]
 
         # Remove unused observation types (i.e., columns of `obs`).
         n_col_keep = ~np.all(np.isnan(obs), axis=0)
         obs = obs[:, n_col_keep]
+        LLI = LLI[:, n_col_keep]
+        SSI = SSI[:, n_col_keep]
         types_list = types_list[n_col_keep]
         types_cnt = len(types_list)
 
@@ -1287,21 +1303,30 @@ class Obs:
         wf2_mat = np.ones((K, J))
         wf2_mat[kk, jj] = wf2[:]
 
-        # Define observation type objects.  Build the visibility matrix.
+        # Define observation, signal strength integer, and loss of lock
+        # indicator attributes.  Build the visibility matrix.
         is_vis = np.full((K, J), False)
         for n_type in range(types_cnt):
-            M = np.full((K, J), FILLER)
-            M[kk, jj] = obs[:, n_type]
-            is_vis += ~np.isnan(M)
-            setattr(self, types_list[n_type], M.copy())
+            obs_mat = np.full((K, J), FILLER)
+            obs_mat[kk, jj] = obs[:, n_type]
+            is_vis += ~np.isnan(obs_mat)
+            setattr(self, types_list[n_type], obs_mat.copy())
+            LLI_mat = np.zeros((K, J), dtype=int)
+            LLI_mat[kk, jj] = LLI[:, n_type]
+            setattr(self, types_list[n_type] + "_LLI", LLI_mat.copy())
+            SSI_mat = np.zeros((K, J), dtype=int)
+            SSI_mat[kk, jj] = SSI[:, n_type]
+            setattr(self, types_list[n_type] + "_SSI", SSI_mat.copy())
 
-        # Define unused observation type objects as `None`.
+        # Define unused observation attributes as `None`.
         all_types = ["C1", "D1", "L1", "S1", "C2", "D2", "L2", "S2",
                 "C5", "D5", "L5", "S5", "C6", "D6", "L6", "S6",
                 "C7", "D7", "L7", "S7", "C8", "D8", "L8", "S8", "P1", "P2"]
         for type_str in all_types:
             if not hasattr(self, type_str):
                 setattr(self, type_str, None)
+                setattr(self, type_str + "_LLI", None)
+                setattr(self, type_str + "_SSI", None)
 
         # Build the prn visibility matrix.  It is the same as the visibility
         # matrix, except that instead of `False` it has FILLER, and instead of
@@ -1310,7 +1335,7 @@ class Obs:
         for j in range(J):
             vis_prn[is_vis[:, j], j] = float(prns[j])
 
-        # Save metadata objects.
+        # Save metadata attributes.
         self.system = system
         self.program = program
         self.author = author
@@ -1331,7 +1356,7 @@ class Obs:
         self.ts_bow = ts_bow
         self.len = len(t_rx)
 
-        # Save array objects.
+        # Save array attributes.
         self.types = types_list.copy()
         self.t = t_rx.copy()
         self.T_os = T_os.copy()
